.vis-3d-settings {
    position: absolute !important;
    top: 0px !important;
}

.vis-3d-stats {
    position: absolute !important;
    top: 0px !important;
}

.neu3dSearchWrapper {
    text-align: right;
    overflow-x: scroll;
    display: flex;
    flex-direction: row;
    justify-content: flex-end;
}

.neu3dSearchInput {
    vertical-align: top;
    flex: 1 1 auto;
    max-width: 350px;
    border: 3px solid #337ab7;
    border-right: none;
    padding: 5px;
    height: 20px;
    outline: none;
    color: #242424;
    margin-top: 2px;
  }

.neu3dSearchButton {
    margin-right: 8px;
    width: 36px;
    height: 32px;
    border: 1px solid #337ab7;
    border-radius: 0 5px 5px 0;
    background: #337ab7;
    text-align: center;
    color: #fff;
    cursor: pointer;
    font-size: 20px;
    margin-top: 2px;
}

.neu3dHintButton {
    width: 36px;
    height: 32px;
    border: 1px solid #337ab7;
    background: #337ab7;
    text-align: center;
    color: #fff;
    border-radius: 5px 0 0 5px;
    cursor: pointer;
    font-size: 20px;
    margin-top: 2px;
}

.jp-FBL-Neu3D-Blocking-Div {
    height: 80px;
    top: calc(50% - 40.5px);
    position: absolute;
    width: 100%;
    background-color: var(--jp-warn-color3);
    opacity: .8;
    display: flex;
    align-items: center;
    justify-content: center;
    font-size: 2rem;
}

.jp-FBL-Neu3D-Container {
    height: calc(100% - 40px);
    width: 100%;
}

.jp-FBL-Neu3D .navbar {
    height: 40px;
    background: #333333;
}