(self.webpackChunk_flybrainlab_neuromynerva=self.webpackChunk_flybrainlab_neuromynerva||[]).push([[915],{9915:(e,r,t)=>{var n=t(4155);r.parse=r.decode=function(e){var r={},t=r,n=null,i=/^\[([^\]]*)\]$|^([^=]+)(=(.*))?$/i;return e.split(/[\r\n]+/g).forEach((function(e,o,c){if(e&&!e.match(/^\s*[;#]/)){var a=e.match(i);if(a){if(void 0!==a[1])return"__proto__"===(n=u(a[1]))?void(t={}):void(t=r[n]=r[n]||{});var s=u(a[2]);if("__proto__"!==s){var f=!a[3]||u(a[4]);switch(f){case"true":case"false":case"null":f=JSON.parse(f)}if(s.length>2&&"[]"===s.slice(-2)){if("__proto__"===(s=s.substring(0,s.length-2)))return;t[s]?Array.isArray(t[s])||(t[s]=[t[s]]):t[s]=[]}Array.isArray(t[s])?t[s].push(f):t[s]=f}}}})),Object.keys(r).filter((function(e,t,n){if(!r[e]||"object"!=typeof r[e]||Array.isArray(r[e]))return!1;var i=o(e),c=r,a=i.pop(),u=a.replace(/\\\./g,".");return i.forEach((function(e,r,t){"__proto__"!==e&&(c[e]&&"object"==typeof c[e]||(c[e]={}),c=c[e])})),(c!==r||u!==a)&&(c[u]=r[e],!0)})).forEach((function(e,t,n){delete r[e]})),r},r.stringify=r.encode=function e(r,t){var n=[],c="";"string"==typeof t?t={section:t,whitespace:!1}:(t=t||{}).whitespace=!0===t.whitespace;var u=t.whitespace?" = ":"=";return Object.keys(r).forEach((function(e,t,o){var s=r[e];s&&Array.isArray(s)?s.forEach((function(r){c+=a(e+"[]")+u+a(r)+"\n"})):s&&"object"==typeof s?n.push(e):c+=a(e)+u+a(s)+i})),t.section&&c.length&&(c="["+a(t.section)+"]"+i+c),n.forEach((function(n,a,u){var s=o(n).join("\\."),f=(t.section?t.section+".":"")+s,l=e(r[n],{section:f,whitespace:t.whitespace});c.length&&l.length&&(c+=i),c+=l})),c},r.safe=a,r.unsafe=u;var i=void 0!==n&&"win32"===n.platform?"\r\n":"\n";function o(e){return e.replace(/\1/g,"LITERAL\\1LITERAL").replace(/\\\./g,"").split(/\./).map((function(e){return e.replace(/\1/g,"\\.").replace(/\2LITERAL\\1LITERAL\2/g,"")}))}function c(e){return'"'===e.charAt(0)&&'"'===e.slice(-1)||"'"===e.charAt(0)&&"'"===e.slice(-1)}function a(e){return"string"!=typeof e||e.match(/[=\r\n]/)||e.match(/^\[/)||e.length>1&&c(e)||e!==e.trim()?JSON.stringify(e):e.replace(/;/g,"\\;").replace(/#/g,"\\#")}function u(e,r){if(!c(e=(e||"").trim())){for(var t=!1,n="",i=0,o=e.length;i<o;i++){var a=e.charAt(i);if(t)-1!=="\\;#".indexOf(a)?n+=a:n+="\\"+a,t=!1;else{if(-1!==";#".indexOf(a))break;"\\"===a?t=!0:n+=a}}return t&&(n+="\\"),n.trim()}"'"===e.charAt(0)&&(e=e.substr(1,e.length-2));try{e=JSON.parse(e)}catch(e){}return e}},4155:e=>{var r,t,n=e.exports={};function i(){throw new Error("setTimeout has not been defined")}function o(){throw new Error("clearTimeout has not been defined")}function c(e){if(r===setTimeout)return setTimeout(e,0);if((r===i||!r)&&setTimeout)return r=setTimeout,setTimeout(e,0);try{return r(e,0)}catch(t){try{return r.call(null,e,0)}catch(t){return r.call(this,e,0)}}}!function(){try{r="function"==typeof setTimeout?setTimeout:i}catch(e){r=i}try{t="function"==typeof clearTimeout?clearTimeout:o}catch(e){t=o}}();var a,u=[],s=!1,f=-1;function l(){s&&a&&(s=!1,a.length?u=a.concat(u):f=-1,u.length&&h())}function h(){if(!s){var e=c(l);s=!0;for(var r=u.length;r;){for(a=u,u=[];++f<r;)a&&a[f].run();f=-1,r=u.length}a=null,s=!1,function(e){if(t===clearTimeout)return clearTimeout(e);if((t===o||!t)&&clearTimeout)return t=clearTimeout,clearTimeout(e);try{t(e)}catch(r){try{return t.call(null,e)}catch(r){return t.call(this,e)}}}(e)}}function p(e,r){this.fun=e,this.array=r}function y(){}n.nextTick=function(e){var r=new Array(arguments.length-1);if(arguments.length>1)for(var t=1;t<arguments.length;t++)r[t-1]=arguments[t];u.push(new p(e,r)),1!==u.length||s||c(h)},p.prototype.run=function(){this.fun.apply(null,this.array)},n.title="browser",n.browser=!0,n.env={},n.argv=[],n.version="",n.versions={},n.on=y,n.addListener=y,n.once=y,n.off=y,n.removeListener=y,n.removeAllListeners=y,n.emit=y,n.prependListener=y,n.prependOnceListener=y,n.listeners=function(e){return[]},n.binding=function(e){throw new Error("process.binding is not supported")},n.cwd=function(){return"/"},n.chdir=function(e){throw new Error("process.chdir is not supported")},n.umask=function(){return 0}}}]);