.table-grid {
display: grid;
grid-template-columns: repeat(auto-fill, minmax(300px, 1fr));
grid-gap: 1px;
border-bottom-color: rgb(221, 221, 221);
border-bottom-width: 1.5px;
border-bottom-style: solid;
}

.table-grid > div {
display: grid;
grid-template-columns: 1fr 2fr;
justify-content: center;
box-sizing: border-box;
border-bottom-color: rgb(221, 221, 221);
border-bottom-width: 1px;
border-bottom-style: solid;
grid-column-gap: 0px;
grid-row-gap: 1px;
align-items: center;
}

.table-grid > div p {
margin: 0px;
padding: 8px;
box-sizing: border-box;
align-self: center;
color: var(--font-color-p);
}
  
.summary-table {
    width: 100%;
    margin: 0;
}

.morphometry-table {
    width: 100%;
    margin: 0;
}

.additional-info-table {
    width: 100%;
    margin: 0;
}

.jp-FBL-Info-sectionContainer {
    display: block;
}

.jp-FBL-Info-sectionContainer.hidden {
    display: none;
}