@import url('base.css');
@import url("processor.css");
@import url("info-widget/info.css");
@import url("info-widget/summary.css");
@import url("master-widget/master.css");
@import url("neu3d-widget/neu3d.css");
@import url("neugfx-widget/neugfx.css");
@import url("template-widget/template.css");