.jp-FBL-Master {
    display: flex;
    flex-direction: column;
    min-width: var(--jp-sidebar-min-width);
    color: var(--jp-ui-font-color1);
    background: var(--jp-layout-color1);
    /* This is needed so that all font sizing of children done in ems is
     * relative to this base size */
    font-size: var(--jp-ui-font-size1);
    overflow: scroll;
}

.jp-FBL-Master-header {
flex: 0 0 auto;
display: flex;
flex-direction: row;
justify-content: flex-end;
}

.jp-FBL-Master-section {
display: flex;
flex: 0 1 auto;
flex-direction: column;
overflow: auto;
}

.jp-FBL-Master-sectionHeader {
flex: 0 0 auto;
align-items: center;
justify-content: space-between;
height: 28px;
display: flex;
border-bottom: var(--jp-border-width) solid var(--jp-border-color2);
margin-top: 8px;
}

.jp-FBL-Master-sectionHeader h2 {
flex: 0 0 auto;
font-weight: 600;
text-transform: uppercase;
letter-spacing: 1px;
font-size: var(--jp-ui-font-size0);
padding: 8px 8px 8px 12px;
margin: 0px;
}

.jp-FBL-Master-sectionHeader .jp-ToolbarButtonComponent {
flex: 0 0 auto;
}

.jp-FBL-Master-sectionContainer {
    flex: 1 1 auto;
    margin: 0;
    padding: 0;
    overflow: auto;
    display: block;
}

.jp-FBL-Master-sectionList {
margin: 0;
padding: 0;
list-style-type: none;
}

.jp-FBL-Master-item {
display: flex;
flex-direction: row;
color: var(--jp-ui-font-color1);
height: var(--jp-private-running-item-height);
line-height: var(--jp-private-running-item-height);
}

.jp-FBL-Master-itemLabel {
font-size: var(--jp-ui-font-size1);
flex: 1 1 auto;
margin-right: 4px;
padding-left: 4px;
text-overflow: ellipsis;
overflow: hidden;
white-space: nowrap;
border-radius: 2px;
transition: background-color 0.1s ease;
}

.jp-FBL-Master-itemLabel:hover {
background-color: var(--jp-layout-color2);
}

.jp-FBL-Master-itemLabel:focus {
background-color: rgba(153, 153, 153, 0.2);
}

.jp-FBL-Master-itemShutdown.jp-mod-styled {
margin: 0px 4px;
color: var(--jp-warn-color1);
font-weight: 500;
background-color: transparent;
height: var(--jp-private-running-shutdown-button-height);
line-height: var(--jp-private-running-shutdown-button-height);
transition: background-color 0.1s ease;
border-radius: 2px;
}

.jp-FBL-Master-itemShutdown.jp-mod-styled:hover {
background-color: rgba(153, 153, 153, 0.1);
}

.jp-FBL-Master-itemShutdown.jp-mod-styled:focus {
border: none;
box-shadow: none;
background-color: rgba(153, 153, 153, 0.2);
}

.jp-FBL-Master-itemDispose.jp-mod-styled {
    margin: 0px 4px;
    color: var(--jp-warn-color1);
    font-size: var(--jp-ui-font-size0);
    font-weight: 500;
    background-color: transparent;
    height: var(--jp-private-running-shutdown-button-height);
    line-height: var(--jp-private-running-shutdown-button-height);
    transition: background-color 0.1s ease;
    border-radius: 2px;
    }
    
.jp-FBL-Master-itemDispose.jp-mod-styled:hover {
    background-color: rgba(153, 153, 153, 0.1);
}
    
.jp-FBL-Master-itemDispose.jp-mod-styled:focus {
    border: none;
    box-shadow: none;
    background-color: rgba(153, 153, 153, 0.2);
}

table.neuron-list-table {
    width: 100%;
}

td.Neuron-Table-Buttons {
    max-width:100% !important;
    white-space:nowrap !important;
}

.jp-FBL-Master-Neu3D-Table-Container {
    display:block;
}
.jp-FBL-Master-Neu3D-Table-Container.hidden {
    display: none;
}

.jp-FBL-Master-Neu3D-Table-Button.hidden {
    display: none;
}

td.Neuron-Table-Buttons {
    display: flex;
    flex-direction: row;
    justify-content: flex-end;
}

.jp-FBL-Master-Collapsible-Section-Header-Buttons {
    display: flex;
    flex-direction: row;
    justify-content: flex-end;
}

.tabulator-col .tabulator-col-title {
    font-size: 10px;
    padding-right: 0px !important;
}

.tabulator-header-filter input {
    font-size: 10px;
}

.jp-FBL-Master-Neu3D-Table .tabulator-col-sorter {
    visibility: hidden;
}

.jp-FBL-Master-Neu3D-Table .tabulator-col[aria-sort="none"] .tabulator-col-title {
    font-weight: normal;
    font-style: normal;
    text-decoration: none;
}

.jp-FBL-Master-Neu3D-Table .tabulator-col[aria-sort="asc"] .tabulator-col-title {
    font-weight: bold;
    font-style: italic;
    text-decoration: overline;
}

.jp-FBL-Master-Neu3D-Table .tabulator-col[aria-sort="desc"] .tabulator-col-title {
    font-weight: bold;
    font-style: oblique;
    text-decoration: underline;
}

.jp-FBL-Master-Neu3D-Table .tabulator-row .tabulator-cell {
    font-size: 0.8em;
}